# Permission Resolver

A zero-dependency RBAC + ABAC permission resolving engine for any Node.js or TypeScript application.

> Easily use it in Express, NestJS, React, or microservices.

---

## Installation

```bash
npm install @ymerncloud/permission-resolver
```

## Usage

```bash

import {
  PermissionResolver,
  Subject,
  Resource,
  PermissionRule,
} from "@merncloud/permission-resolver";

const rules: PermissionRule[] = [
  {
    resource: "document",
    action: "read",
    allowedRoles: ["editor"],
    attributeCondition: {
      and: [
        { "==": [{ var: "subject.team" }, { var: "resource.team" }] },
        { ">=": [{ var: "subject.level" }, 2] },
      ],
    },
  },
];

const subject: Subject = {
  id: "user-1",
  roles: ["editor"],
  attributes: { team: "alpha", level: 3 },
};

const resource: Resource = {
  id: "doc-1",
  type: "document",
  attributes: { team: "alpha" },
};

const resolver = new PermissionResolver(rules);

const canRead = resolver.can(subject, resource, "read");

console.log("✅ Can Read:", canRead);


```

### Loading Permissions from Database

To keep the library framework-agnostic and DB-independent, you can pass your own async loader function.

```ts
import { createResolverFromSource } from "@yourorg/permission-resolver";
import { Pool } from "pg";

const pool = new Pool({
  /* connection */
});

const loadRules = async () => {
  const client = await pool.connect();
  const res = await client.query("SELECT * FROM permissions");
  client.release();

  return res.rows.map((row) => ({
    resource: row.resource,
    action: row.action,
    allowedRoles: row.allowed_roles,
    attributeCondition: row.attribute_condition,
  }));
};

const resolver = await createResolverFromSource(loadRules);
```

Sample App Implementation

```ts
import { createResolverFromSource } from "@merncloud/permission-resolver";
import { Pool } from "pg";

// PostgreSQL connection
const pool = new Pool({
  host: "localhost",
  port: 5432,
  user: "your_postgres_user",
  password: "your_password",
  database: "your_db",
});

// Loader function to fetch rules
const loadRules = async () => {
  const client = await pool.connect();
  const res = await client.query("SELECT * FROM permissions");
  client.release();

  return res.rows.map((row) => ({
    resource: row.resource,
    action: row.action,
    allowedRoles: row.allowed_roles,
    attributeCondition: row.attribute_condition, // should be JSON object
  }));
};

const doSomething = async () => {
  const resolver = await createResolverFromSource(loadRules);

  const subject = {
    id: "user-1",
    roles: ["admin"],
    attributes: {
      team: "marketing",
      level: 3,
    },
  };

  const resource = {
    id: "doc-123",
    type: "document",
    attributes: {
      team: "marketing",
    },
  };

  const canRead = resolver.can(subject, resource, "read");
  const canDelete = resolver.can(subject, resource, "delete");
  const canWrite = resolver.can(subject, resource, "write");

  console.log("Can read:", canRead);
  console.log("Can delete:", canDelete);
  console.log("Can write:", canWrite);

  await pool.end();
};

doSomething();
```

Build your db for sample app

```sql

-- Drop if exists
DROP TABLE IF EXISTS permissions;

-- Create permissions table
CREATE TABLE permissions (
  id SERIAL PRIMARY KEY,
  resource TEXT NOT NULL,
  action TEXT NOT NULL,
  allowed_roles JSONB NOT NULL,
  attribute_condition JSONB
);

-- Seed sample permissions

-- Rule 1: Editors and admins can read documents from their own team
INSERT INTO permissions (resource, action, allowed_roles, attribute_condition)
VALUES (
  'document',
  'read',
  '["editor", "admin"]',
  '{
    "==": [
      { "var": "subject.team" },
      { "var": "resource.team" }
    ]
  }'
);

-- Rule 2: Admins can write documents if subject level >= 3
INSERT INTO permissions (resource, action, allowed_roles, attribute_condition)
VALUES (
  'document',
  'write',
  '["admin"]',
  '{
    "and": [
      { "==": [{ "var": "subject.team" }, { "var": "resource.team" }] },
      { ">=": [{ "var": "subject.level" }, 3] }
    ]
  }'
);

-- Rule 3: Managers can delete users only from the same region
INSERT INTO permissions (resource, action, allowed_roles, attribute_condition)
VALUES (
  'user',
  'delete',
  '["manager"]',
  '{
    "==": [
      { "var": "subject.region" },
      { "var": "resource.region" }
    ]
  }'
);


```
